@@ -70,6 +70,7 @@ internal class Api
 			WM_CHAR = 0x0102,
 			WM_SYSKEYDOWN = 0x0104,
 			WM_SYSKEYUP = 0x0105,
+			WM_SYSCHAR = 0x0106,
 			WM_SYSCOMMAND = 0x112,
 			WM_TIMER = 0x0113,
 			WM_MOUSEMOVE = 0x0200,
@@ -393,6 +394,8 @@ internal class Api
 		MA_NOACTIVATEANDEAT = 4
 	}
 
+	public const uint CBM_INIT = 0x04;
+
 	[StructLayout(LayoutKind.Sequential,CharSet=CharSet.Ansi)]
 	public struct WNDCLASS 
 	{
@@ -910,7 +913,7 @@ internal class Api
 
 	[DllImport("gdi32")]
 	public static extern IntPtr CreateDIBitmap( IntPtr hdc, byte[] lpbmih, uint fdwInit, byte[] lpbInit, byte[] lpbmi, uint fuUsage);
-
+	
 	[DllImport("user32")]
 	public static extern int SetWindowLongA( IntPtr hWnd, SetWindowLongType nIndex, WindowStyle dwNewLong);
 
@@ -931,6 +934,9 @@ internal class Api
 
 	[DllImport("gdi32")]
 	public static extern IntPtr CreateRectRgn(int nLeftRect, int nTopRect, int nRightRect, int nBottomRect);
+	
+	[DllImport("gdi32")]
+	public static extern bool SetRectRgn(IntPtr hrgn, int nLeftRect, int nTopRect, int nRightRect, int nBottomRect);
 
 	[DllImport("gdi32")]
 	public static extern int CombineRgn( IntPtr hrgnDest, IntPtr hrgnSrc1, IntPtr hrgnSrc2, RegionCombineMode fnCombineMode);
@@ -1025,6 +1031,9 @@ internal class Api
 	[DllImport("user32")]
 	public static extern IntPtr CreateIconIndirect(ref ICONINFO piconinfo);
 
+	[DllImport("gdi32")]
+	public static extern bool DeleteDC(IntPtr hdc);
+
 }//Api
 
 }
