@@ -211,7 +211,7 @@ internal sealed class DrawingPopupWindow : PopupWindow, IToolkitWindow
 	// Force an update of all invalidated regions.
 	void IToolkitWindow.Update()
 			{
-				Update();
+				Update(false);
 				Display.Flush();
 			}
 
@@ -370,13 +370,12 @@ internal sealed class DrawingPopupWindow : PopupWindow, IToolkitWindow
 			{
 				if(sink != null)
 				{
+					System.Drawing.Region clip = DrawingWindow.RegionToDrawingRegion
+									(graphics.ExposeRegion); 
 					DrawingGraphics g = new DrawingGraphics(toolkit, graphics);
 					using(System.Drawing.Graphics gr =
-							  ToolkitManager.CreateGraphics(g))
+							  ToolkitManager.CreateGraphics(g, clip))
 					{
-						gr.SetClip(DrawingWindow.RegionToDrawingRegion
-							(graphics.ExposeRegion),
-							CombineMode.Replace);
 						sink.ToolkitExpose(gr);
 					}
 				}
