@@ -212,7 +212,7 @@ internal sealed class DrawingWindow : InputOutputWidget, IToolkitWindow
 	// Force an update of all invalidated regions.
 	void IToolkitWindow.Update()
 			{
-				Update();
+				Update(false);
 				Display.Flush();
 			}
 
@@ -585,12 +585,11 @@ internal sealed class DrawingWindow : InputOutputWidget, IToolkitWindow
 			{
 				if(sink != null)
 				{
+					System.Drawing.Region clip = RegionToDrawingRegion(graphics.ExposeRegion);
 					DrawingGraphics g = new DrawingGraphics(toolkit, graphics);
 					using(System.Drawing.Graphics gr =
-								ToolkitManager.CreateGraphics(g))
+								ToolkitManager.CreateGraphics(g, clip))
 					{
-						gr.SetClip(RegionToDrawingRegion(graphics.ExposeRegion),
-								   CombineMode.Replace);
 						sink.ToolkitExpose(gr);
 					}
 				}
@@ -623,25 +622,8 @@ internal sealed class DrawingWindow : InputOutputWidget, IToolkitWindow
 				}
 				else if(cursorType == ToolkitCursorType.Default)
 				{
-					// Try to inherit the desktop's cursor for the default.
-					Widget parent = widget.Parent;
-					while(parent != null)
-					{
-						if(parent.Cursor != null)
-						{
-							break;
-						}
-						parent = parent.Parent;
-					}
-					if(parent != null)
-					{
-						widget.Cursor = new Cursor(CursorType.XC_left_ptr);
-					}
-					else
-					{
-						widget.Cursor =
-							new Cursor(CursorType.XC_inherit_parent);
-					}
+					// Set the default cursor to something matching X.
+					widget.Cursor = new Cursor(CursorType.XC_left_ptr);
 				}
 				else
 				{
