@@ -30,7 +30,13 @@ using System.Runtime.Serialization;
 [Serializable]
 public class ImageListStreamer : ISerializable
 {
+	internal ImageList.ImageCollection images;
 	// TODO
+
+	internal ImageListStreamer(ImageList imageList)
+	{
+		// TODO
+	}
 	private ImageListStreamer(SerializationInfo info, StreamingContext context)
 	{
 	}
@@ -40,6 +46,14 @@ public class ImageListStreamer : ISerializable
 		// TODO
 	}
 
+	internal ImageList.ImageCollection Images
+	{
+		get
+		{
+			return images;
+		}
+	}
+
 }; // class ImageListStreamer
 
 #endif // CONFIG_SERIALIZATION
