@@ -81,7 +81,7 @@ namespace System.Windows.Forms
 		// Tab events
 		public event DrawItemEventHandler DrawItem;
 		public event EventHandler SelectedIndexChanged;
-		public TabControl() : base()
+		public TabControl()
 		{
 			selectedIndex = -1;
 
@@ -120,19 +120,26 @@ namespace System.Windows.Forms
 		protected virtual void OnSelectedIndexChanged(EventArgs e)
 		{
 			if (selectedIndex == -1 || selectedIndex >= TabCount)
+			{
 				return;
+			}
 			SuspendLayout();
-			positionInfo = null;
 			if (prevSelectedIndex > -1)
 			{
 				GetChildByIndex( prevSelectedIndex ).Visible = false;
 			}
-			GetChildByIndex( selectedIndex ).Visible = true;
+
+			Control selectedPage = GetChildByIndex( selectedIndex );
+			selectedPage.Visible = true;
+			SetTabPageBounds();
 			
 			if (SelectedIndexChanged != null)
+			{
 				SelectedIndexChanged( this, EventArgs.Empty );
+			}
 			ResumeLayout();
 			InvalidateTabs();
+
 		}
 
 		protected void RemoveAll()
@@ -259,11 +266,18 @@ namespace System.Windows.Forms
 		{
 			get
 			{
-				return tabPageCollection[selectedIndex] as TabPage;
+				if (selectedIndex == -1)
+				{
+					return null;
+				}
+				else
+				{
+					return tabPageCollection[selectedIndex] as TabPage;
+				}
 			}
 			set
 			{
-				SelectedIndex = tabPageCollection.GetChildIndex(value);
+				SelectedIndex = tabPageCollection.IndexOf(value);
 			}
 		}
 
@@ -309,17 +323,17 @@ namespace System.Windows.Forms
 		
 		protected override Control.ControlCollection CreateControlsInstance()
 		{
-			return new TabPageCollection( this );
+			return new TabChildControls( this );
 		}
 
 		
 		// Collection of child control TabPages.
-		public class TabPageCollection : Control.ControlCollection
+		public class TabChildControls : Control.ControlCollection
 		{
 			// Owning tab control.
 			private TabControl tabOwner;
 
-			public TabPageCollection(TabControl owner) : base(owner)
+			public TabChildControls(TabControl owner) : base(owner)
 			{
 				this.tabOwner = owner;
 			}
@@ -328,6 +342,182 @@ namespace System.Windows.Forms
 			{
 				control.Visible = false;
 				base.Add(control);
+				tabOwner.InvalidateTabs	();
+				tabOwner.SetTabPageBounds();
+				if (tabOwner.SelectedTab == control)
+				{
+					control.Visible = true;
+				}
+			}
+
+			[TODO]
+			public override void Remove(Control control)
+			{
+				// TODO
+				base.Remove(control);
+			}
+
+		};
+
+		// Collection of child control TabPages.
+		public class TabPageCollection : ICollection, IEnumerable, IList
+		{
+			// Owning tab control.
+			private TabControl tabOwner;
+
+			public TabPageCollection(TabControl owner)
+			{
+				this.tabOwner = owner;
+			}
+
+			[TODO]
+			public TabPage this[int idx]
+			{
+				get
+				{
+					return (TabPage)(tabOwner.Controls[idx]);
+				}
+				set
+				{
+					// TODO
+				}
+			}
+
+			public void Add(TabPage control)
+			{
+				if(control == null)
+				{
+					throw new ArgumentNullException("control");
+				}
+				tabOwner.Controls.Add(control);
+			}
+
+			public void Remove(TabPage control)
+			{
+				if(control == null)
+				{
+					throw new ArgumentNullException("control");
+				}
+				tabOwner.Controls.Remove(control);
+			}
+
+			public void AddRange(TabPage[] pages)
+			{
+				if(pages == null)
+				{
+					throw new ArgumentException("pages");
+				}
+				foreach(TabPage page in pages)
+				{
+					Add(page);
+				}
+			}
+
+			public bool Controls(TabPage control)
+			{
+				return tabOwner.Controls.Contains(control);
+			}
+
+			public int IndexOf(TabPage control)
+			{
+				return tabOwner.Controls.IndexOf(control);
+			}
+
+			// Implement the ICollection interface.
+			void ICollection.CopyTo(Array array, int index)
+			{
+				tabOwner.Controls.CopyTo(array, index);
+			}
+			public int Count
+			{
+				get
+				{
+					return tabOwner.Controls.Count;
+				}
+			}
+			bool ICollection.IsSynchronized
+			{
+				get
+				{
+					return false;
+				}
+			}
+			Object ICollection.SyncRoot
+			{
+				get
+				{
+					return this;
+				}
+			}
+
+			// Implement the IEnumerable interface.
+			public IEnumerator GetEnumerator()
+			{
+				return tabOwner.Controls.GetEnumerator();
+			}
+
+			// Implement the IList interface.
+			int IList.Add(Object value)
+			{
+				if(!(value is TabPage))
+				{
+					throw new ArgumentException("value");
+				}
+				Add((TabPage)value);
+				return ((IList)(tabOwner.Controls)).IndexOf(value);
+			}
+			public void Clear()
+			{
+				tabOwner.RemoveAll();
+			}
+			bool IList.Contains(Object value)
+			{
+				return ((IList)(tabOwner.Controls)).Contains(value);
+			}
+			int IList.IndexOf(Object value)
+			{
+				return ((IList)(tabOwner.Controls)).IndexOf(value);
+			}
+			void IList.Insert(int index, Object value)
+			{
+				throw new NotSupportedException();
+			}
+			void IList.Remove(Object value)
+			{
+				if(!(value is TabPage))
+				{
+					throw new ArgumentException("value");
+				}
+				Remove((TabPage)value);
+			}
+			public void RemoveAt(int index)
+			{
+				Remove(this[index]);
+			}
+			bool IList.IsFixedSize
+			{
+				get
+				{
+					return false;
+				}
+			}
+			bool IList.IsReadOnly
+			{
+				get
+				{
+					return false;
+				}
+			}
+			Object IList.this[int index]
+			{
+				get
+				{
+					return this[index];
+				}
+				set
+				{
+					this[index] = (TabPage)value;
+				}
 			}
 
 		};
@@ -339,6 +529,16 @@ namespace System.Windows.Forms
 			base.OnPaint (e);
 		}
 
+		protected override void OnPaintBackground(PaintEventArgs e)
+		{
+			base.OnPaintBackground (e);
+			using (Brush back = new SolidBrush(BackColor))
+			{
+				e.Graphics.FillRectangle(back, 0, 0, Width, Height);
+			}
+		}
+
+
 		private void Draw(Graphics g)
 		{
 			// Exclude the moveButtons from the draw to prevent flicker.
@@ -774,14 +974,17 @@ namespace System.Windows.Forms
 		public int SelectedIndex {
 			get
 			{
-				return selectedIndex;
+				if (selectedIndex == -1)
+				{
+					return 0;
+				}
+				else
+				{
+					return selectedIndex;
+				}
 			}
 			set
 			{
-				if (value == -1)
-				{
-					value = 0;
-				}
 				if (value != selectedIndex)
 				{
 					prevSelectedIndex = selectedIndex;
@@ -890,7 +1093,6 @@ namespace System.Windows.Forms
 				if (moveButtonsTabOffset > 0)
 				{
 					moveButtonsTabOffset--;
-					positionInfo = null;
 					InvalidateTabs();
 				}
 			}
@@ -900,7 +1102,6 @@ namespace System.Windows.Forms
 				if (moveButtonsTabOffset < tabPageCollection.Count - 1 && moveButtonsCovered)
 				{
 					moveButtonsTabOffset++;
-					positionInfo = null;
 					InvalidateTabs();
 				}
 			}
@@ -911,8 +1112,20 @@ namespace System.Windows.Forms
 				if (newSelectedIndex > -1)
 				{
 					SelectedIndex = newSelectedIndex;
-					// Makes sure that when tabbing off the tab into the child controls that the tab order resets.
-					ActiveControl = this;
+					
+					// Handle focus.
+					if (!Focused)
+					{
+						if (!SelectedTab.SelectNextControl(null, true, true, false, false))
+						{
+							IContainerControl container = Parent.GetContainerControl();
+							if (container != null)
+							{
+								container.ActiveControl = this;
+							}
+						}
+					}
+
 				}
 			}
 			base.OnMouseDown (e);
@@ -933,6 +1146,15 @@ namespace System.Windows.Forms
 			base.OnEnter (e);
 		}
 
+		protected override bool ProcessKeyPreview(ref Message m)
+		{
+			if (ProcessKeyEventArgs(ref m))
+			{
+				return true; 
+			}
+			return base.ProcessKeyPreview(ref m); 
+		}
+
 
 		// Find which tab the mouse is over
 		private int GetMouseOverTab(int x, int y)
@@ -1068,6 +1290,7 @@ namespace System.Windows.Forms
 
 		internal void InvalidateTabs()
 		{
+			positionInfo = null;
 			if (!IsHandleCreated)
 				return;
 			Rectangle bounds = GetTabsBounds();
@@ -1107,7 +1330,7 @@ namespace System.Windows.Forms
 			// Force the positions of the tabs to update if the size changes
 			if ((specified & BoundsSpecified.Size) != 0)
 			{
-				positionInfo = null;
+				InvalidateTabs();
 				SetTabPageBounds();
 				Invalidate(false);
 			}
@@ -1116,10 +1339,6 @@ namespace System.Windows.Forms
 		protected override void OnCreateControl()
 		{
 			base.OnCreateControl();
-			if (selectedIndex == -1)
-			{
-				SelectedIndex = 0;
-			}
 			SetTabPageBounds();
 		}
 
