@@ -25,6 +25,7 @@ namespace System.Windows.Forms.Themes
 
 using System.Drawing;
 using System.Drawing.Drawing2D;
+using System.Drawing.Toolkit;
 using System.Windows.Forms;
 
 // This class implements the default theme functionality for the
@@ -40,6 +41,17 @@ internal class DefaultThemePainter : IThemePainter
 	protected Size gridSpacing = Size.Empty;
 	protected Brush gridBrush = null;
 
+	// Draw a simple bitmap-based glyph within a rectangle.
+	private static void DrawGlyph
+				(Graphics graphics, int x, int y, int width, int height,
+				 byte[] bits, int bitsWidth, int bitsHeight, Color color)
+			{
+				x += (width - bitsWidth) / 2;
+				y += (height - bitsHeight) / 2;
+				ToolkitManager.DrawGlyph
+					(graphics, x, y, bits, bitsWidth, bitsHeight, color);
+			}
+
 	// Get the width of the specified border type
 	public virtual int GetBorderWidth(BorderStyle border)
 			{
@@ -88,7 +100,18 @@ internal class DefaultThemePainter : IThemePainter
 					{
 						pen = new Pen(color, 1.0f);
 						pen.EndCap = LineCap.Square;
-						graphics.DrawRectangle(pen, bounds);
+						Rectangle r = new Rectangle(bounds.X,
+							bounds.Y, bounds.Width - 1, bounds.Height - 1);
+						graphics.DrawRectangle(pen, r);
+						pen.Color = ControlPaint.LightLight(color);
+						graphics.DrawLine(pen, bounds.X + 1,
+							bounds.Y + bounds.Height - 1,
+							bounds.X + bounds.Width - 1,
+							bounds.Y + bounds.Height - 1);
+						graphics.DrawLine(pen, bounds.X + bounds.Width - 1,
+							bounds.Y + bounds.Height - 2,
+							bounds.X + bounds.Width - 1,
+							bounds.Y + 1);
 						pen.Dispose();
 					}
 					break;
@@ -883,12 +906,135 @@ internal class DefaultThemePainter : IThemePainter
 			}
 
 	// Draw a caption button control.
-	[TODO]
 	public virtual void DrawCaptionButton
 				(Graphics graphics, int x, int y, int width, int height,
 				 CaptionButton button, ButtonState state)
 			{
-				// TODO
+				Color color;
+
+				// Draw the border on the button.
+				DrawButton(graphics, x, y, width, height, state,
+						   SystemColors.ControlText, SystemColors.Control,
+						   false);
+
+				// Adjust the glyph position if necessary.
+				if((state & ButtonState.Pushed) != 0)
+				{
+					++x;
+					++y;
+				}
+
+				// Draw the inactive drop shadow glyph on the button.
+				if((state & ButtonState.Inactive) != 0)
+				{
+					color = SystemColors.ControlLightLight;
+					switch(button)
+					{
+						case CaptionButton.Close:
+						{
+							DrawGlyph(graphics, x + 1, y + 1, width, height,
+									  Glyphs.close_button_bits,
+									  Glyphs.close_button_width,
+									  Glyphs.close_button_height, color);
+						}
+						break;
+
+						case CaptionButton.Minimize:
+						{
+							DrawGlyph(graphics, x + 1, y + 1, width, height,
+									  Glyphs.minimize_button_bits,
+									  Glyphs.minimize_button_width,
+									  Glyphs.minimize_button_height, color);
+						}
+						break;
+
+						case CaptionButton.Maximize:
+						{
+							DrawGlyph(graphics, x + 1, y + 1, width, height,
+									  Glyphs.maximize_button_bits,
+									  Glyphs.maximize_button_width,
+									  Glyphs.maximize_button_height, color);
+						}
+						break;
+
+						case CaptionButton.Restore:
+						{
+							DrawGlyph(graphics, x + 1, y + 1, width, height,
+									  Glyphs.restore_button_bits,
+									  Glyphs.restore_button_width,
+									  Glyphs.restore_button_height, color);
+						}
+						break;
+
+						case CaptionButton.Help:
+						{
+							DrawGlyph(graphics, x + 1, y + 1, width, height,
+									  Glyphs.help_button_bits,
+									  Glyphs.help_button_width,
+									  Glyphs.help_button_height, color);
+						}
+						break;
+					}
+				}
+
+				// Determine the color to use for the glyph.
+				if((state & ButtonState.Inactive) != 0)
+				{
+					color = SystemColors.ControlDark;
+				}
+				else
+				{
+					color = SystemColors.ControlText;
+				}
+
+				// Draw the glyph on the button.
+				switch(button)
+				{
+					case CaptionButton.Close:
+					{
+						DrawGlyph(graphics, x, y, width, height,
+								  Glyphs.close_button_bits,
+								  Glyphs.close_button_width,
+								  Glyphs.close_button_height, color);
+					}
+					break;
+
+					case CaptionButton.Minimize:
+					{
+						DrawGlyph(graphics, x, y, width, height,
+								  Glyphs.minimize_button_bits,
+								  Glyphs.minimize_button_width,
+								  Glyphs.minimize_button_height, color);
+					}
+					break;
+
+					case CaptionButton.Maximize:
+					{
+						DrawGlyph(graphics, x, y, width, height,
+								  Glyphs.maximize_button_bits,
+								  Glyphs.maximize_button_width,
+								  Glyphs.maximize_button_height, color);
+					}
+					break;
+
+					case CaptionButton.Restore:
+					{
+						DrawGlyph(graphics, x, y, width, height,
+								  Glyphs.restore_button_bits,
+								  Glyphs.restore_button_width,
+								  Glyphs.restore_button_height, color);
+					}
+					break;
+
+					case CaptionButton.Help:
+					{
+						DrawGlyph(graphics, x, y, width, height,
+								  Glyphs.help_button_bits,
+								  Glyphs.help_button_width,
+								  Glyphs.help_button_height, color);
+					}
+					break;
+				}
 			}
 
 	// Draw a progress bar control.
@@ -946,10 +1092,82 @@ internal class DefaultThemePainter : IThemePainter
 			}
 
 	// Draw a check box control.
-	// TODO: antialiased checkmark 
 	public virtual void DrawCheckBox(Graphics graphics, int x, int y,
 								     int width, int height, ButtonState state)
 			{
+				// Draw the border around the outside of the checkbox.
+				if((state & ButtonState.Flat) == 0)
+				{
+					ControlPaint.DrawBorder3D
+						(graphics, x, y, width, height,
+						 Border3DStyle.Sunken, Border3DSide.All);
+					x += 2;
+					y += 2;
+					width -= 4;
+					height -= 4;
+				}
+				else
+				{
+					if((state & ButtonState.Inactive) != 0)
+					{
+						graphics.DrawRectangle
+							(SystemPens.ControlDark, x, y,
+							 width - 1, height - 1);
+					}
+					else
+					{
+						graphics.DrawRectangle
+							(SystemPens.ControlText, x, y,
+							 width - 1, height - 1);
+					}
+					x += 1;
+					y += 1;
+					width -= 2;
+					height -= 2;
+				}
+
+				// Fill the background of the checkbox.
+				if((state & (ButtonState.Inactive | ButtonState.Pushed)) != 0)
+				{
+					graphics.FillRectangle
+						(SystemBrushes.Control, x, y, width, height);
+				}
+				else if((state & (ButtonState)0x10000) != 0)
+				{
+					// Checkbox is in the indeterminate state.
+					graphics.FillRectangle
+						(SystemBrushes.ControlLightLight, x, y, width, height);
+				}
+				else
+				{
+					graphics.FillRectangle
+						(SystemBrushes.Window, x, y, width, height);
+				}
+
+				// Draw the checkmark in the checkbox.
+				if((state & ButtonState.Checked) != 0)
+				{
+					Color color;
+					if((state & ButtonState.Inactive) != 0)
+					{
+						color = SystemColors.ControlDark;
+					}
+					else if((state & (ButtonState)0x10000) != 0)
+					{
+						// Checkbox is in the indeterminate state.
+						color = SystemColors.ControlDark;
+					}
+					else
+					{
+						color = SystemColors.ControlText;
+					}
+					DrawGlyph(graphics, x, y, width, height,
+							  Glyphs.checkmark_bits,
+							  Glyphs.checkmark_width,
+							  Glyphs.checkmark_height, color);
+				}
+
+#if false	// May need later for non-standard glyph sizes.
 				Brush brush = new SolidBrush(Color.White);
 				ControlPaint.DrawBorder3D(graphics, x, y, width, height, Border3DStyle.Sunken, Border3DSide.All);
 				graphics.FillRectangle(brush, x + 2, y + 2, width - 2, height - 2);
@@ -967,6 +1185,7 @@ internal class DefaultThemePainter : IThemePainter
 					blackBrush.Dispose();
 					pen.Dispose();
 				}
+#endif
 			}
 
 	// Draw a combo box's drop down button control.
@@ -974,18 +1193,48 @@ internal class DefaultThemePainter : IThemePainter
 								        int width, int height,
 									    ButtonState state)
 			{
-				Border3DStyle style = (state == ButtonState.Pushed) ?
-					Border3DStyle.Sunken : Border3DStyle.Raised;
-				ControlPaint.DrawBorder3D(graphics, x, y, width, height, style, Border3DSide.All);
-				graphics.FillRectangle(SystemBrushes.Control, x + 2, y + 2, width - 4, height - 4);
-				x += width / 2;
-				y += height * 2 / 3;
+				// Draw the border part of the button.
+				DrawButton(graphics, x, y, width, height, state,
+				           SystemColors.ControlText, SystemColors.Control,
+				           false);
+
+				// Get the drawing color and adjust for the pressed state.
+				Color color;
+				if((state & ButtonState.Pushed) != 0)
+				{
+					color = SystemColors.ControlText;
+					++x;
+					++y;
+				}
+				else if((state & ButtonState.Inactive) != 0)
+				{
+					color = SystemColors.ControlDark;
+					DrawGlyph(graphics, x + 1, y + 1, width, height,
+							  Glyphs.drop_down_arrow_bits,
+							  Glyphs.drop_down_arrow_width,
+							  Glyphs.drop_down_arrow_height,
+							  SystemColors.ControlLightLight);
+				}
+				else
+				{
+					color = SystemColors.ControlText;
+				}
+
+				// Draw the glyph on the button.
+				DrawGlyph(graphics, x, y, width, height,
+						  Glyphs.drop_down_arrow_bits,
+						  Glyphs.drop_down_arrow_width,
+						  Glyphs.drop_down_arrow_height, color);
+
+#if false	// May need later for non-standard glyph sizes.
+				x += width / 2 + 2;
+				y += height * 2 / 3 + 2;
 				Brush arrow = (state == ButtonState.Inactive) ?
 					SystemBrushes.ControlDark : SystemBrushes.ControlText;
 				graphics.FillPolygon(arrow, new PointF[]
-					{
-						new Point(x, y), new Point(x - 4, y - 4), new Point(x + 4, y - 4)
-					});
+						{new Point(x, y), new Point(x - 4, y - 4),
+						 new Point(x + 4, y - 4)});
+#endif
 			}
 
 	// Draw a container grab handle.
@@ -1273,12 +1522,19 @@ internal class DefaultThemePainter : IThemePainter
 						int spaceY = gridSpacing.Height;
 						using (Bitmap bmp = new Bitmap(spaceX+1,spaceY+1))
 						{
+							for (int y = 0; y < bmp.Height; y++)
+							{
+								for (int x = 0; x < bmp.Width; x++)
+								{
+									bmp.SetPixel(x, y, backColor);
+								}
+							}
 							bmp.SetPixel(0, 0, color);
 							gridBrush = new TextureBrush(bmp);
+							graphics.FillRectangle(gridBrush, area);
 						}
 					}
 				}
-				graphics.FillRectangle(gridBrush, area);
 			}
 
 	// Draw an image in its disabled state.
@@ -1287,7 +1543,8 @@ internal class DefaultThemePainter : IThemePainter
 				(Graphics graphics, Image image,
 				 int x, int y, Color background)
 			{
-				// TODO
+				// TODO: create the greyed version of the image.
+				graphics.DrawImage(image, x, y);
 			}
 
 	// Draw a locked selection frame.
@@ -1316,21 +1573,52 @@ internal class DefaultThemePainter : IThemePainter
 			}
 
 	// Draw a menu glyph.
-	[TODO]
 	public virtual void DrawMenuGlyph
 				(Graphics graphics, int x, int y, int width,
 				 int height, MenuGlyph glyph)
 			{
-				// TODO
+				switch(glyph)
+				{
+					case MenuGlyph.Arrow:
+					{
+						DrawGlyph(graphics, x, y, width, height,
+								  Glyphs.menu_arrow_bits,
+								  Glyphs.menu_arrow_width,
+								  Glyphs.menu_arrow_height,
+								  SystemColors.MenuText);
+					}
+					break;
+
+					case MenuGlyph.Checkmark:
+					{
+						DrawGlyph(graphics, x, y, width, height,
+								  Glyphs.menu_checkmark_bits,
+								  Glyphs.menu_checkmark_width,
+								  Glyphs.menu_checkmark_height,
+								  SystemColors.MenuText);
+					}
+					break;
+
+					case MenuGlyph.Bullet:
+					{
+						DrawGlyph(graphics, x, y, width, height,
+								  Glyphs.menu_bullet_bits,
+								  Glyphs.menu_bullet_width,
+								  Glyphs.menu_bullet_height,
+								  SystemColors.MenuText);
+						
+					}
+					break;
+				}
 			}
 
 	// Draw a three-state check box control.
-	[TODO]
 	public virtual void DrawMixedCheckBox
 				(Graphics graphics, int x, int y, int width,
 				 int height, ButtonState state)
 			{
-				// TODO
+				DrawCheckBox(graphics, x, y, width, height,
+							 state | (ButtonState)0x10000);
 			}
 
 	// Draw a radio button control.
@@ -1340,20 +1628,72 @@ internal class DefaultThemePainter : IThemePainter
 				 ButtonState state, Color foreColor, Color backColor,
 				 Brush backgroundBrush)
 			{
-				// fill in the background
+				// Fill in the background.
 				graphics.FillRectangle(backgroundBrush, x, y, width, height);
 
+				// Collect up the colors that we need to draw the button.
 				Color lightlight = ControlPaint.LightLight(backColor);
 				Color darkdark = ControlPaint.DarkDark(backColor);
 				Color light = ControlPaint.Light(backColor);
 				Color dark = ControlPaint.Dark(backColor);
-
-				Color fillColor = lightlight;
-				if ((state & ButtonState.Inactive) != 0)
+				Color fillColor;
+				Color bulletColor;
+				if((state & ButtonState.Inactive) != 0)
 				{
 					fillColor = light;
+					bulletColor = dark;
+				}
+				else if((state & ButtonState.Pushed) != 0)
+				{
+					fillColor = light;
+					bulletColor = foreColor;
+				}
+				else
+				{
+					fillColor = lightlight;
+					bulletColor = foreColor;
 				}
 
+				// Draw the glyphs that make up the components.
+				int radio_width = Glyphs.radio_width;
+				int radio_height = Glyphs.radio_height;
+				if((state & ButtonState.Flat) == 0)
+				{
+					DrawGlyph(graphics, x, y, width, height,
+							  Glyphs.radio1_bits,
+							  radio_width, radio_height, dark);
+					DrawGlyph(graphics, x, y, width, height,
+							  Glyphs.radio2_bits,
+							  radio_width, radio_height, darkdark);
+					DrawGlyph(graphics, x, y, width, height,
+							  Glyphs.radio3_bits,
+							  radio_width, radio_height, lightlight);
+					DrawGlyph(graphics, x, y, width, height,
+							  Glyphs.radio4_bits,
+							  radio_width, radio_height, light);
+					DrawGlyph(graphics, x, y, width, height,
+							  Glyphs.radio5_bits,
+							  radio_width, radio_height, fillColor);
+				}
+				else
+				{
+					DrawGlyph(graphics, x, y, width, height,
+							  Glyphs.rflat1_bits,
+							  radio_width, radio_height, bulletColor);
+					DrawGlyph(graphics, x, y, width, height,
+							  Glyphs.rflat2_bits,
+							  radio_width, radio_height, fillColor);
+				}
+
+				// Draw the glyph for the checked state.
+				if((state & ButtonState.Checked) != 0)
+				{
+					DrawGlyph(graphics, x, y, width, height,
+							  Glyphs.radio_bullet_bits,
+							  radio_width, radio_height, bulletColor);
+				}
+
+#if false	// May need later for non-standard glyph sizes.
 				// fill in center area
 				using (Brush b = new SolidBrush(fillColor))
 				{
@@ -1412,22 +1752,7 @@ internal class DefaultThemePainter : IThemePainter
 						}
 					}
 				}
-			}
-
-	// Draw a reversible frame.
-	[TODO]
-	public virtual void DrawReversibleFrame
-				(Rectangle rectangle, Color backColor, FrameStyle style)
-			{
-				// TODO
-			}
-
-	// Draw a reversible line.
-	[TODO]
-	public virtual void DrawReversibleLine
-				(Point start, Point end, Color backColor)
-			{
-				// TODO
+#endif
 			}
 
 	private Brush hatchBrush;
@@ -1450,8 +1775,10 @@ internal class DefaultThemePainter : IThemePainter
 					backColor != hatchBackColor))
 						hatchBrush.Dispose();
 				if (hatchBrush == null)
-					hatchBrush = new HatchBrush(HatchStyle.Percent50,
-						   backColor, foreColor);
+					hatchBrush = new HatchBrush
+						  (HatchStyle.Percent50,
+						   SystemColors.ScrollBar,
+						   SystemColors.ControlLightLight);
 
 				Color color;
 				if (backColor.GetBrightness() > 0.5f)
@@ -1550,6 +1877,109 @@ internal class DefaultThemePainter : IThemePainter
 				           state,
 				           foreColor, backColor,
 				           false);
+
+				// Get the drawing color and adjust for the pressed state.
+				Color color;
+				Color inactiveColor = ControlPaint.LightLight(backColor);
+				if((state & ButtonState.Pushed) != 0)
+				{
+					color = foreColor;
+					++x;
+					++y;
+				}
+				else if((state & ButtonState.Inactive) != 0)
+				{
+					color = ControlPaint.Dark(backColor);
+				}
+				else
+				{
+					color = foreColor;
+				}
+
+				// Draw the inactive drop-down glyph for the button.
+				if((state & ButtonState.Inactive) != 0)
+				{
+					switch (button)
+					{
+						case ScrollButton.Up:
+						{
+							DrawGlyph(graphics, x + 1, y + 1, width, height,
+									  Glyphs.up_arrow_bits,
+									  Glyphs.up_arrow_width,
+									  Glyphs.up_arrow_height, inactiveColor);
+						}
+						break;
+	
+						case ScrollButton.Down:
+						{
+							DrawGlyph(graphics, x + 1, y + 1, width, height,
+									  Glyphs.down_arrow_bits,
+									  Glyphs.down_arrow_width,
+									  Glyphs.down_arrow_height, inactiveColor);
+						}
+						break;
+	
+						case ScrollButton.Left:
+						{
+							DrawGlyph(graphics, x + 1, y + 1, width, height,
+									  Glyphs.left_arrow_bits,
+									  Glyphs.left_arrow_width,
+									  Glyphs.left_arrow_height, inactiveColor);
+						}
+						break;
+	
+						case ScrollButton.Right:
+						{
+							DrawGlyph(graphics, x + 1, y + 1, width, height,
+									  Glyphs.right_arrow_bits,
+									  Glyphs.right_arrow_width,
+									  Glyphs.right_arrow_height, inactiveColor);
+						}
+						break;
+					}
+				}
+
+				// Draw the glyph for the button.
+				switch (button)
+				{
+					case ScrollButton.Up:
+					{
+						DrawGlyph(graphics, x, y, width, height,
+								  Glyphs.up_arrow_bits,
+								  Glyphs.up_arrow_width,
+								  Glyphs.up_arrow_height, color);
+					}
+					break;
+
+					case ScrollButton.Down:
+					{
+						DrawGlyph(graphics, x, y, width, height,
+								  Glyphs.down_arrow_bits,
+								  Glyphs.down_arrow_width,
+								  Glyphs.down_arrow_height, color);
+					}
+					break;
+
+					case ScrollButton.Left:
+					{
+						DrawGlyph(graphics, x, y, width, height,
+								  Glyphs.left_arrow_bits,
+								  Glyphs.left_arrow_width,
+								  Glyphs.left_arrow_height, color);
+					}
+					break;
+
+					case ScrollButton.Right:
+					{
+						DrawGlyph(graphics, x, y, width, height,
+								  Glyphs.right_arrow_bits,
+								  Glyphs.right_arrow_width,
+								  Glyphs.right_arrow_height, color);
+					}
+					break;
+				}
+
+#if false	// May need later for non-standard glyph sizes.
 				x += 2; // skip border
 				y += 2; // skip border
 				width -= 4; // skip border
@@ -1649,6 +2079,7 @@ internal class DefaultThemePainter : IThemePainter
 				{
 					graphics.DrawPolygon(pen,glyph);
 				}
+#endif
 			}
 
 	// Draw a selection frame.
@@ -1670,37 +2101,18 @@ internal class DefaultThemePainter : IThemePainter
 			}
 
 	// Draw a list box
-	public void DrawListBox(Graphics graphics, BorderStyle border,
+	public void DrawListBox(Graphics graphics,
 		int x, int y, int width, int height, bool corner,
 		int cornerHeight, int cornerWidth)
 			{
-				// Repaint the border
-				switch(border)
-				{
-					case BorderStyle.Fixed3D:
-						ControlPaint.DrawBorder3D(
-							graphics, 
-							new Rectangle(0, 0, width, height),
-							Border3DStyle.Sunken);
-						break;
-						
-					case BorderStyle.FixedSingle:
-						ControlPaint.DrawBorder(
-							graphics, 
-							new Rectangle(0, 0, width - 1, height - 1), 
-							SystemColors.WindowText, 
-							ButtonBorderStyle.Solid);
-						break;
-				}
-				
 				// If both scroll bars are visible, then paint the region
 				// that is blank where the two meet (lower-right corner)
 				if(corner)
 				{
 					graphics.FillRectangle(
 						new SolidBrush(SystemColors.Control),
-						width - this.GetBorderWidth(border) - cornerWidth,
-						height - this.GetBorderWidth(border) - cornerHeight,
+						width - cornerWidth,
+						height - cornerHeight,
 						cornerWidth,
 						cornerHeight);
 				}
@@ -1722,12 +2134,17 @@ internal class DefaultThemePainter : IThemePainter
 				brush.Dispose();
 			}
 
-	// Draw a filled reversible rectangle.
-	[TODO]
-	public virtual void FillReversibleRectangle
-				(Rectangle rectangle, Color backColor)
+	// Draw a menu separator line.
+	public virtual void DrawSeparator(Graphics graphics, Rectangle rectangle)
 			{
-				// TODO
+				int x = rectangle.X + 2;
+				int width = rectangle.Width - 4;
+				int y = rectangle.Y + (rectangle.Height / 2);
+				graphics.DrawLine
+					(SystemPens.ControlDark, x, y, x + width - 1, y);
+				graphics.DrawLine
+					(SystemPens.ControlLightLight, x, y + 1,
+					 x + width - 1, y + 1);
 			}
 
 }; // class DefaultThemePainter
