@@ -68,14 +68,23 @@ public class HttpWebRequest : WebRequest
 	// so that it is accessible to the nested classes
 	private Stream outStream=null;
 	private WebResponse response=null;
-	private const String format="ddd, dd MMM yyyy HH*:mm:ss GMTzz";//HTTP
+	private const String format="ddd, dd MMM yyyy HH:mm:ss 'GMT'zz";//HTTP
+
+	private HttpController controller=null;
 
 	internal HttpWebRequest(Uri uri)
 	{
-		this.address=uri;
 		this.originalUri=uri;
-		this.isSecured=String.Equals(uri.Scheme,Uri.UriSchemeHttps);
 		this.method="GET";
+		this.headers.SetInternal ("User-Agent","DotGNU Portable.net");
+		SetAddress(uri);
+	}
+
+	internal void SetAddress(Uri uri)
+	{
+		CheckHeadersSent(); /* I know this never gets called , but better safe than sorry */
+		this.address=uri;
+		this.isSecured=String.Equals(uri.Scheme,Uri.UriSchemeHttps);
 		this.headers.SetInternal ("Host", uri.Authority);
 		this.headers.SetInternal ("Date", DateTime.Now.ToUniversalTime().ToString(format));
 	}
@@ -167,6 +176,7 @@ public class HttpWebRequest : WebRequest
 			throw new WebException(" not allowed for " + this.Method);
 		if(outStream==null)
 		{
+			if(preAuthenticate) AddHttpAuthHeaders(null);
 			outStream=new HttpStream(this);
 		}
 		return outStream;
@@ -175,19 +185,35 @@ public class HttpWebRequest : WebRequest
 	//need some doubts cleared
 	public override WebResponse GetResponse()
 	{
+		HttpWebResponse httpResponse;
+		WebRequest newRequest;
+
 		if(response!=null) return response;
+		
 		if(outStream==null)
 		{
+			if(preAuthenticate) AddHttpAuthHeaders(null);
 			outStream=new HttpStream(this);
 			outStream.Flush();
 			// which is the response stream as well 
 		}
-		if(this.response==null)
+
+		httpResponse=new HttpWebResponse(this,this.outStream);
+		this.response=httpResponse;
+
+		/* here it always is a HttpWebResponse , need to use a Factory actually 
+		 * When that happens, change the design */
+		
+		newRequest=Controller.Recurse(this, httpResponse);
+		
+		/* this is the tricky recursion thing */ 
+		this.response=newRequest.GetResponse(); 
+		
+		this.haveResponse=true; // I hope this is correct
+		if(outStream!=null)
 		{
-			this.response=new HttpWebResponse(this,this.outStream);
-			this.haveResponse=true; // I hope this is correct
+			(outStream as HttpStream).ContentLength=response.ContentLength;
 		}
-		(outStream as HttpStream).ContentLength=response.ContentLength;
 		return this.response; 
 	}
 
@@ -459,7 +485,7 @@ public class HttpWebRequest : WebRequest
 		}
 	}
 
-	public override bool PreAuthenticate 
+	public override bool PreAuthenticate
 	{
 		get
 		{
@@ -489,11 +515,14 @@ public class HttpWebRequest : WebRequest
 		}
 	}
 
-	[TODO]
 	public override IWebProxy Proxy 
 	{
 		get
 		{
+			if(this.proxy==null)
+			{
+				this.proxy=GlobalProxySelection.Select;
+			}
 			return this.proxy;
 		} 
 		set
@@ -617,6 +646,123 @@ public class HttpWebRequest : WebRequest
 		return false;
 	}
 
+	internal bool AddHttpAuthHeaders(String challenge)
+	{
+		ICredentials cred;
+		String challengeType="Basic"; /* default */
+		Authorization auth;
+		
+		if(credentials==null) return false;
+		if(challenge==null && !preAuthenticate) 
+		{
+			return false; /* TODO : throw an exception here ? */
+		}
+		else if (challenge!=null)
+		{
+			int len=challenge.IndexOf(' ');
+			challengeType=( len==-1) ? challenge : 
+						challenge.Substring(0,len);
+		}
+		cred=credentials.GetCredential(this.Address,challengeType);
+		if(cred==null)
+		{
+			return false; /* TODO : throw an exception here ? */
+		}
+
+		if(preAuthenticate)
+		{
+			auth=AuthenticationManager.PreAuthenticate(this, 
+							cred);
+		}
+		else
+		{
+			auth=AuthenticationManager.Authenticate(challenge,
+							this, cred);
+		}
+
+		if(auth==null)
+		{
+			return false; /* TODO : throw an exception here ? */
+		}
+		this.Headers["Authorization"]=auth.Message;
+		return true;
+	}
+
+	internal bool AddProxyAuthHeaders(String challenge)
+	{
+		ICredentials cred;
+		String challengeType="Basic"; /* default */
+		Authorization auth;
+		
+		if(proxy==null) return false;
+		
+		if(proxy.Credentials==null) return false;
+
+		if(challenge==null)
+		{
+			throw new ArgumentNullException("challenge");
+		}
+		else
+		{
+			int len=challenge.IndexOf(' ');
+			challengeType=( len==-1) ? challenge : 
+						challenge.Substring(0,len);
+		}
+
+		cred=proxy.Credentials.GetCredential(this.Address,challengeType);
+		
+		if(cred==null)
+		{
+			return false; /* TODO : throw an exception here ? */
+		}
+
+		/* I would have added a PreAuthenticate option for this as well 
+		 * But MS or ECMA doesn't . So I won't -- Gopal */
+
+		auth=AuthenticationManager.Authenticate(challenge,
+							this, cred);
+		if(auth==null)
+		{
+			return false; /* TODO : throw an exception here ? */
+		}
+
+		this.Headers["Proxy-Authorization"]=auth.Message;
+		return true;
+	}
+
+	/* Re-use an HTTP Webrequest 
+	 * Note: use with great care ,might be buggy -- Gopal 
+	 * The basic assumption is that this WebRequest and the stream is of no further
+	 * use. So beware , content will be lost for the last call .
+	 * However the response will be retained if it is an incomplete Reset*/
+	internal void ResetRequest(bool complete) 
+	{
+		if(outStream!=null) 
+		{
+			/* theoretically I should use the same KeepAlive stream for subsequent requests
+			 * but the HttpStream needs a bit of hacking in that case */
+			outStream.Close();
+		}
+		headerSent=false;
+		if(complete)
+		{
+			response=null;
+		}
+		outStream=null;
+	}
+
+	private HttpController Controller
+	{
+			get
+			{
+					if(controller==null) controller=new HttpController(this);
+					return this.controller;
+			}
+	}
+
+	/* Note: the next cycle of refactoring should split this single
+	 * stream class into two http streams . Current hack works, but
+	 * only *just* */
 	private class HttpStream : Stream
 	{	
 		private HttpWebRequest request;
@@ -635,6 +781,15 @@ public class HttpWebRequest : WebRequest
 			SendHeaders();
 		}
 
+		~HttpStream()
+		{
+			if(underlying!=null)
+			{
+				underlying.Close();
+				underlying=null;
+			}
+		}
+
 		// Stub out all stream functionality.
 		public override void Flush() 
 		{
@@ -651,6 +806,7 @@ public class HttpWebRequest : WebRequest
 		
 		public override int ReadByte() 
 		{
+			if(contentLength<=0) return -1;
 			int retval = underlying.ReadByte();
 			contentLength-=1;
 			return retval;
@@ -676,6 +832,11 @@ public class HttpWebRequest : WebRequest
 			underlying.WriteByte(value);
 		}
 
+		public override void Close()
+		{
+			/* Nothing */
+		}
+
 		public override bool CanRead 
 		{
 			get 
@@ -738,20 +899,24 @@ public class HttpWebRequest : WebRequest
 		private static Socket OpenSocket(HttpWebRequest req)
 		{
 			IPAddress ip=null;
-			if(req.Address.HostNameType == UriHostNameType.Dns)
+			Uri nextHop=(req.Proxy!=null) ?
+			       	req.Proxy.GetProxy(req.Address) : req.Address;
+			
+			if(nextHop.HostNameType == UriHostNameType.Dns)
 			{
-				ip=Dns.Resolve(req.Address.Host).AddressList[0];
+				ip=Dns.Resolve(nextHop.Host).AddressList[0];
 			}
-			else if(req.Address.HostNameType == UriHostNameType.IPv4 ||
-					req.Address.HostNameType == UriHostNameType.IPv6)
+			else if(nextHop.HostNameType == UriHostNameType.IPv4 ||
+					nextHop.HostNameType == UriHostNameType.IPv6)
 			{
-				ip=IPAddress.Parse(req.Address.Host);
+				ip=IPAddress.Parse(nextHop.Host);
 			}
-			IPEndPoint ep = new IPEndPoint(ip,req.Address.Port);
+			IPEndPoint ep = new IPEndPoint(ip,nextHop.Port);
 			Socket server=new 
 					Socket(ip.AddressFamily, SocketType.Stream,
 							ProtocolType.Tcp);
 			server.Connect(ep);
+			/* TODO: Tunnel via Proxy before starting SSL */
 			return server;
 		}
 
@@ -762,6 +927,7 @@ public class HttpWebRequest : WebRequest
 			{
 #if CONFIG_SSL
 				SecureConnection secured=new SecureConnection();
+				/*TODO: suspected memory hold for SecureConnection */
 				Stream retval=secured.OpenStream(sock);
 				return retval;
 #else
@@ -779,10 +945,28 @@ public class HttpWebRequest : WebRequest
 			StreamWriter writer=new StreamWriter(this);
 			request.headerSent=true; 
 			/* fake it before sending to allow for atomicity */
-			String requestString= request.Method+" "+
+			String requestString= null;
+			if(request.Proxy!=null)
+			{
+				if(request.Proxy.Credentials != null &&
+					request.Method != "GET" && 
+					request.Method != "HEAD")
+				{
+					request.AddProxyAuthHeaders("Basic");
+				}		
+						
+				requestString= request.Method+" "+
+					request.Address+
+					" HTTP/"+request.protocolVersion.Major+
+					"."+request.protocolVersion.Minor+"\r\n";
+			}
+			else
+			{
+				requestString= request.Method+" "+
 					request.Address.PathAndQuery+
 					" HTTP/"+request.protocolVersion.Major+
 					"."+request.protocolVersion.Minor+"\r\n";
+			}
 			writer.Write(requestString);
 			writer.Write(request.Headers.ToString());
 			writer.Write("\r\n");// terminating CRLF
@@ -821,6 +1005,137 @@ public class HttpWebRequest : WebRequest
 		}
 	}
 #endif
+
+	/* this class forms the controller of the http life cycle
+	 * of OK,Forbiddens and redirects */
+	private class HttpController
+	{
+		/* I know it's a bad practice to have seperate states for each thing
+		 * but I'll fix it later :-) */
+		private enum HttpAuthState
+		{
+			NoAuth,
+			Trying,
+			OK,
+			Failed
+		}
+		
+		private HttpAuthState proxy;
+		private HttpAuthState http;
+		private int redirections;
+
+		public HttpController(HttpWebRequest request)
+		{
+			if(request.PreAuthenticate)
+			{
+				this.http=HttpAuthState.Trying;
+			}
+			else
+			{
+				this.http=HttpAuthState.NoAuth;
+			}
+			this.proxy=HttpAuthState.NoAuth;
+			redirections=0;
+		}
+		/* returns this WebRequest or a new one.
+		 * If nothing can be done, it will return the same 'ol 
+		 * request.
+		 * Note: will change substantially soon*/
+		public WebRequest Recurse(HttpWebRequest request,HttpWebResponse response)
+		{
+			HttpStatusCode code=response.StatusCode;
+			switch(code)
+			{
+				case HttpStatusCode.OK:
+				{
+					if(http==HttpAuthState.Trying) http=HttpAuthState.OK;
+					if(proxy==HttpAuthState.Trying) proxy=HttpAuthState.OK;
+					return request;
+				}
+				break; /* never reached */
+				
+				case HttpStatusCode.ProxyAuthenticationRequired:
+				{
+						String challenge=null;
+
+						if(proxy==HttpAuthState.Trying || proxy==HttpAuthState.Failed) 
+						{
+							proxy=HttpAuthState.Failed;
+							return request;
+						}
+						challenge=response.Headers["Proxy-Authenticate"];
+						request.ResetRequest(false);
+						if(request.AddProxyAuthHeaders(challenge))
+						{
+							request.ResetRequest(true);
+							proxy=HttpAuthState.Trying;
+							return request;
+						}
+						else
+						{
+							proxy=HttpAuthState.Failed;
+							return request;
+						}
+				}
+				break;
+
+				case HttpStatusCode.Forbidden:
+				{
+						String challenge=null;
+
+						if(http==HttpAuthState.Trying || http==HttpAuthState.Failed) 
+						{
+							http=HttpAuthState.Failed;
+							return request;
+						}
+						challenge=response.Headers["WWW-Authenticate"];
+						request.ResetRequest(false);
+						if(request.AddHttpAuthHeaders(challenge))
+						{
+							request.ResetRequest(true);
+							http=HttpAuthState.Trying;
+							return request;
+						}
+						else
+						{
+							http=HttpAuthState.Failed;
+							return request;
+						}
+				}
+				break;
+
+				case HttpStatusCode.Redirect:
+				case HttpStatusCode.Moved :
+				case HttpStatusCode.MultipleChoices :
+				case HttpStatusCode.SeeOther :
+				case HttpStatusCode.TemporaryRedirect:
+				/* case HttpStatusCode.Found : // Duplicate */
+				/* case HttpStatusCode.MovedPermanently : // Duplicate */
+				{
+						Uri newUri=response.ResponseUri;
+						if(request.allowAutoRedirect &&	newUri!=null &&
+							redirections<request.MaximumAutomaticRedirections &&
+							(request.Method=="GET" || request.Method=="HEAD"))
+						{
+							/* ok redirect it */
+							request.ResetRequest(true);
+							request.SetAddress(newUri);
+							redirections++;
+							return request;
+						}
+				}
+				break;
+
+			}
+			
+			return request;
+		}
+
+		public override String ToString()
+		{
+				return "(Proxy="+proxy.ToString()+",Http="+http.ToString()+")";
+		}
+	}
 }//class
 
 }//namespace
