@@ -381,6 +381,7 @@ namespace System.Text.RegularExpressions {
 						if (Eval (Mode.Match, ref ptr, pc + 1))
 							goto Pass;
 					
+						this.repeat = current;
 						goto Fail;
 					}
 
@@ -781,7 +782,11 @@ namespace System.Text.RegularExpressions {
 		}
 
 		private void Backtrack (int cp) {
-			Debug.Assert (cp > mark_start, "Regex", "Attempt to backtrack forwards");
+			// #if PNET -- this assertion is incorrect and will fail
+			// on perfectly legitimate regexes.  The assertion doesn't
+			// typically fire on Mono because the "Conditional" attribute
+			// removes the call to Debug.Assert, and everything works fine.
+			//Debug.Assert (cp > mark_start, "Regex", "Attempt to backtrack forwards");
 
 			for (int i = 0; i < groups.Length; ++ i) {
 				int m = groups [i];
