@@ -206,15 +206,15 @@ public class Assembly : IClrProgramItem
 				{
 					return null;
 				}
-				else if(type == null)
+				else if(type != null)
 				{
-					return GetManifestResourceStream(type.Namespace + "." +
-													 name);
-				}
-				else
-				{
-					return GetManifestResourceStream(name);
+					String nspace = type.Namespace;
+					if(nspace != null && nspace != String.Empty)
+					{
+						return GetManifestResourceStream(nspace + "." + name);
+					}
 				}
+				return GetManifestResourceStream(name);
 			}
 
 #else // ECMA_COMPAT
